@@ -13,19 +13,29 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and* limitations under the License.*
 """
 
+# For python 2.7
+from __future__ import print_function
 from botocore import session, awsrequest, auth
+import logging
+import sys
 
 from . import CerberusClientException, CLIENT_VERSION
 from .util import throw_if_bad_response, post_with_retry
+from collections import OrderedDict
+
+
+logger = logging.getLogger(__name__)
+
 
 class AWSAuth(object):
     """Class to authenticate with an IAM Role"""
     HEADERS = {"Content-Type": "application/json", "X-Cerberus-Client": "CerberusPythonClient/" + CLIENT_VERSION}
 
-    def __init__(self, cerberus_url, region, aws_session=None):
+    def __init__(self, cerberus_url, region, aws_session=None, verbose=None):
         self.cerberus_url = cerberus_url
         self.region = region
         self.aws_session = aws_session
+        self.verbose = verbose
 
     def _get_v4_signed_headers(self):
         """Returns V4 signed get-caller-identity request headers"""
@@ -39,7 +49,7 @@ class AWSAuth(object):
         readonly_credentials = creds.get_frozen_credentials()
 
         # hardcode get-caller-identity request
-        data = {'Action': 'GetCallerIdentity', 'Version': '2011-06-15'}
+        data = OrderedDict((('Action', 'GetCallerIdentity'), ('Version', '2011-06-15')))
         url = 'https://sts.{}.amazonaws.com/'.format(self.region)
         request_object = awsrequest.AWSRequest(method='POST', url=url, data=data)
 
@@ -58,6 +68,8 @@ class AWSAuth(object):
 
         token = resp.json()['client_token']
         iam_principal_arn = resp.json()['metadata']['aws_iam_principal_arn']
-        print('Successful authenticated with Cerberus as {}'.format(iam_principal_arn))
+        if self.verbose:
+            print('Successfully authenticated with Cerberus as {}'.format(iam_principal_arn), file=sys.stderr)
+        logger.info('Successfully authenticated with Cerberus as {}'.format(iam_principal_arn))
 
         return token
